from . import simpl

def generate_vertex_coordinates(data_container_array, selected_data_container_name = None, coordinate_array_path = None, observer = None) -> int:
  """
  GenerateVertexCoordinates

  :param data_container_array | DataContainerArray
  :param selected_data_container_name | DataArrayPath SelectedDataContainerName
  :param coordinate_array_path | DataArrayPath CoordinateArrayPath
  :param observer | Observer
  """

  filter = simpl.GenerateVertexCoordinates()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_container_name is not None:
    filter.SelectedDataContainerName = selected_data_container_name
  if coordinate_array_path is not None:
    filter.CoordinateArrayPath = coordinate_array_path
  filter.execute()
  return filter.ErrorCode

def array_calculator(data_container_array, selected_attribute_matrix = None, infix_equation = None, calculated_array = None, units = None, scalar_type = None, observer = None) -> int:
  """
  ArrayCalculator

  :param data_container_array | DataContainerArray
  :param selected_attribute_matrix | DataArrayPath SelectedAttributeMatrix
  :param infix_equation | QString InfixEquation
  :param calculated_array | DataArrayPath CalculatedArray
  :param units | ArrayCalculator::AngleUnits Units
  :param scalar_type | SIMPL::ScalarTypes::Type ScalarType
  :param observer | Observer
  """

  filter = simpl.ArrayCalculator()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_attribute_matrix is not None:
    filter.SelectedAttributeMatrix = selected_attribute_matrix
  if infix_equation is not None:
    filter.InfixEquation = infix_equation
  if calculated_array is not None:
    filter.CalculatedArray = calculated_array
  if units is not None:
    filter.Units = units
  if scalar_type is not None:
    filter.ScalarType = scalar_type
  filter.execute()
  return filter.ErrorCode

def combine_attribute_arrays(data_container_array, selected_data_array_paths = None, stacked_data_array_name = None, normalize_data = None, observer = None) -> int:
  """
  CombineAttributeArrays

  :param data_container_array | DataContainerArray
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param stacked_data_array_name | QString StackedDataArrayName
  :param normalize_data | bool NormalizeData
  :param observer | Observer
  """

  filter = simpl.CombineAttributeArrays()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  if stacked_data_array_name is not None:
    filter.StackedDataArrayName = stacked_data_array_name
  if normalize_data is not None:
    filter.NormalizeData = normalize_data
  filter.execute()
  return filter.ErrorCode

def combine_attribute_matrices(data_container_array, first_attribute_matrix_path = None, second_attribute_matrix_path = None, first_index_array_path = None, second_index_array_path = None, combined_attribute_matrix_name = None, new_index_array_name = None, observer = None) -> int:
  """
  CombineAttributeMatrices

  :param data_container_array | DataContainerArray
  :param first_attribute_matrix_path | DataArrayPath FirstAttributeMatrixPath
  :param second_attribute_matrix_path | DataArrayPath SecondAttributeMatrixPath
  :param first_index_array_path | DataArrayPath FirstIndexArrayPath
  :param second_index_array_path | DataArrayPath SecondIndexArrayPath
  :param combined_attribute_matrix_name | QString CombinedAttributeMatrixName
  :param new_index_array_name | QString NewIndexArrayName
  :param observer | Observer
  """

  filter = simpl.CombineAttributeMatrices()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if first_attribute_matrix_path is not None:
    filter.FirstAttributeMatrixPath = first_attribute_matrix_path
  if second_attribute_matrix_path is not None:
    filter.SecondAttributeMatrixPath = second_attribute_matrix_path
  if first_index_array_path is not None:
    filter.FirstIndexArrayPath = first_index_array_path
  if second_index_array_path is not None:
    filter.SecondIndexArrayPath = second_index_array_path
  if combined_attribute_matrix_name is not None:
    filter.CombinedAttributeMatrixName = combined_attribute_matrix_name
  if new_index_array_name is not None:
    filter.NewIndexArrayName = new_index_array_name
  filter.execute()
  return filter.ErrorCode

def conditional_set_value(data_container_array, selected_array_path = None, conditional_array_path = None, replace_value = None, observer = None) -> int:
  """
  ConditionalSetValue

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param conditional_array_path | DataArrayPath ConditionalArrayPath
  :param replace_value | double ReplaceValue
  :param observer | Observer
  """

  filter = simpl.ConditionalSetValue()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if conditional_array_path is not None:
    filter.ConditionalArrayPath = conditional_array_path
  if replace_value is not None:
    filter.ReplaceValue = replace_value
  filter.execute()
  return filter.ErrorCode

def convert_color_to_gray_scale(data_container_array, conversion_algorithm = None, color_weights = None, color_channel = None, input_data_array_vector = None, create_new_attribute_matrix = None, output_attribute_matrix_name = None, output_array_prefix = None, observer = None) -> int:
  """
  ConvertColorToGrayScale

  :param data_container_array | DataContainerArray
  :param conversion_algorithm | int ConversionAlgorithm
  :param color_weights | FloatVec3Type ColorWeights
  :param color_channel | int ColorChannel
  :param input_data_array_vector | std::vector<DataArrayPath> InputDataArrayVector
  :param create_new_attribute_matrix | bool CreateNewAttributeMatrix
  :param output_attribute_matrix_name | QString OutputAttributeMatrixName
  :param output_array_prefix | QString OutputArrayPrefix
  :param observer | Observer
  """

  filter = simpl.ConvertColorToGrayScale()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if conversion_algorithm is not None:
    filter.ConversionAlgorithm = conversion_algorithm
  if color_weights is not None:
    filter.ColorWeights = color_weights
  if color_channel is not None:
    filter.ColorChannel = color_channel
  if input_data_array_vector is not None:
    filter.InputDataArrayVector = input_data_array_vector
  if create_new_attribute_matrix is not None:
    filter.CreateNewAttributeMatrix = create_new_attribute_matrix
  if output_attribute_matrix_name is not None:
    filter.OutputAttributeMatrixName = output_attribute_matrix_name
  if output_array_prefix is not None:
    filter.OutputArrayPrefix = output_array_prefix
  filter.execute()
  return filter.ErrorCode

def convert_data(data_container_array, scalar_type = None, output_array_name = None, selected_cell_array_path = None, observer = None) -> int:
  """
  ConvertData

  :param data_container_array | DataContainerArray
  :param scalar_type | SIMPL::NumericTypes::Type ScalarType
  :param output_array_name | QString OutputArrayName
  :param selected_cell_array_path | DataArrayPath SelectedCellArrayPath
  :param observer | Observer
  """

  filter = simpl.ConvertData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if scalar_type is not None:
    filter.ScalarType = scalar_type
  if output_array_name is not None:
    filter.OutputArrayName = output_array_name
  if selected_cell_array_path is not None:
    filter.SelectedCellArrayPath = selected_cell_array_path
  filter.execute()
  return filter.ErrorCode

def extract_vertex_geometry(data_container_array, array_handling = None, selected_data_container_name = None, included_data_array_paths = None, vertex_data_container_name = None, observer = None) -> int:
  """
  ExtractVertexGeometry

  :param data_container_array | DataContainerArray
  :param array_handling | int ArrayHandling
  :param selected_data_container_name | DataArrayPath SelectedDataContainerName
  :param included_data_array_paths | std::vector<DataArrayPath> IncludedDataArrayPaths
  :param vertex_data_container_name | DataArrayPath VertexDataContainerName
  :param observer | Observer
  """

  filter = simpl.ExtractVertexGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if array_handling is not None:
    filter.ArrayHandling = array_handling
  if selected_data_container_name is not None:
    filter.SelectedDataContainerName = selected_data_container_name
  if included_data_array_paths is not None:
    filter.IncludedDataArrayPaths = included_data_array_paths
  if vertex_data_container_name is not None:
    filter.VertexDataContainerName = vertex_data_container_name
  filter.execute()
  return filter.ErrorCode

def copy_feature_array_to_element_array(data_container_array, selected_feature_array_path = None, feature_ids_array_path = None, created_array_name = None, observer = None) -> int:
  """
  CopyFeatureArrayToElementArray

  :param data_container_array | DataContainerArray
  :param selected_feature_array_path | DataArrayPath SelectedFeatureArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param created_array_name | QString CreatedArrayName
  :param observer | Observer
  """

  filter = simpl.CopyFeatureArrayToElementArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_feature_array_path is not None:
    filter.SelectedFeatureArrayPath = selected_feature_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if created_array_name is not None:
    filter.CreatedArrayName = created_array_name
  filter.execute()
  return filter.ErrorCode

def create_attribute_matrix(data_container_array, created_attribute_matrix = None, attribute_matrix_type = None, tuple_dimensions = None, observer = None) -> int:
  """
  CreateAttributeMatrix

  :param data_container_array | DataContainerArray
  :param created_attribute_matrix | DataArrayPath CreatedAttributeMatrix
  :param attribute_matrix_type | int AttributeMatrixType
  :param tuple_dimensions | DynamicTableData TupleDimensions
  :param observer | Observer
  """

  filter = simpl.CreateAttributeMatrix()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if created_attribute_matrix is not None:
    filter.CreatedAttributeMatrix = created_attribute_matrix
  if attribute_matrix_type is not None:
    filter.AttributeMatrixType = attribute_matrix_type
  if tuple_dimensions is not None:
    filter.TupleDimensions = tuple_dimensions
  filter.execute()
  return filter.ErrorCode

def create_data_array(data_container_array, scalar_type = None, number_of_components = None, new_array = None, initialization_type = None, initialization_value = None, initialization_range = None, observer = None) -> int:
  """
  CreateDataArray

  :param data_container_array | DataContainerArray
  :param scalar_type | SIMPL::ScalarTypes::Type ScalarType
  :param number_of_components | int NumberOfComponents
  :param new_array | DataArrayPath NewArray
  :param initialization_type | int InitializationType
  :param initialization_value | QString InitializationValue
  :param initialization_range | FPRangePair InitializationRange
  :param observer | Observer
  """

  filter = simpl.CreateDataArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if scalar_type is not None:
    filter.ScalarType = scalar_type
  if number_of_components is not None:
    filter.NumberOfComponents = number_of_components
  if new_array is not None:
    filter.NewArray = new_array
  if initialization_type is not None:
    filter.InitializationType = initialization_type
  if initialization_value is not None:
    filter.InitializationValue = initialization_value
  if initialization_range is not None:
    filter.InitializationRange = initialization_range
  filter.execute()
  return filter.ErrorCode

def create_data_container(data_container_array, data_container_name = None, observer = None) -> int:
  """
  CreateDataContainer

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param observer | Observer
  """

  filter = simpl.CreateDataContainer()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  filter.execute()
  return filter.ErrorCode

def create_feature_array_from_element_array(data_container_array, cell_feature_attribute_matrix_name = None, selected_cell_array_path = None, created_array_name = None, feature_ids_array_path = None, observer = None) -> int:
  """
  CreateFeatureArrayFromElementArray

  :param data_container_array | DataContainerArray
  :param cell_feature_attribute_matrix_name | DataArrayPath CellFeatureAttributeMatrixName
  :param selected_cell_array_path | DataArrayPath SelectedCellArrayPath
  :param created_array_name | QString CreatedArrayName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param observer | Observer
  """

  filter = simpl.CreateFeatureArrayFromElementArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_feature_attribute_matrix_name is not None:
    filter.CellFeatureAttributeMatrixName = cell_feature_attribute_matrix_name
  if selected_cell_array_path is not None:
    filter.SelectedCellArrayPath = selected_cell_array_path
  if created_array_name is not None:
    filter.CreatedArrayName = created_array_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  filter.execute()
  return filter.ErrorCode

def create_image_geometry(data_container_array, selected_data_container = None, dimensions = None, origin = None, spacing = None, observer = None) -> int:
  """
  CreateImageGeometry

  :param data_container_array | DataContainerArray
  :param selected_data_container | DataArrayPath SelectedDataContainer
  :param dimensions | IntVec3Type Dimensions
  :param origin | FloatVec3Type Origin
  :param spacing | FloatVec3Type Spacing
  :param observer | Observer
  """

  filter = simpl.CreateImageGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_container is not None:
    filter.SelectedDataContainer = selected_data_container
  if dimensions is not None:
    filter.Dimensions = dimensions
  if origin is not None:
    filter.Origin = origin
  if spacing is not None:
    filter.Spacing = spacing
  filter.execute()
  return filter.ErrorCode

def create_grid_montage(data_container_array, montage_name = None, grid_selection = None, data_container_name_list = None, observer = None) -> int:
  """
  CreateGridMontage

  :param data_container_array | DataContainerArray
  :param montage_name | QString MontageName
  :param grid_selection | DataContainerGrid GridSelection
  :param data_container_name_list | QStringList DataContainerNameList
  :param observer | Observer
  """

  filter = simpl.CreateGridMontage()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if montage_name is not None:
    filter.MontageName = montage_name
  if grid_selection is not None:
    filter.GridSelection = grid_selection
  if data_container_name_list is not None:
    filter.DataContainerNameList = data_container_name_list
  filter.execute()
  return filter.ErrorCode

def create_string_array(data_container_array, number_of_components = None, new_array = None, initialization_value = None, observer = None) -> int:
  """
  CreateStringArray

  :param data_container_array | DataContainerArray
  :param number_of_components | int NumberOfComponents
  :param new_array | DataArrayPath NewArray
  :param initialization_value | QString InitializationValue
  :param observer | Observer
  """

  filter = simpl.CreateStringArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if number_of_components is not None:
    filter.NumberOfComponents = number_of_components
  if new_array is not None:
    filter.NewArray = new_array
  if initialization_value is not None:
    filter.InitializationValue = initialization_value
  filter.execute()
  return filter.ErrorCode

def copy_object(data_container_array, object_to_copy = None, data_container_to_copy = None, attribute_matrix_to_copy = None, attribute_array_to_copy = None, copied_object_name = None, observer = None) -> int:
  """
  CopyObject

  :param data_container_array | DataContainerArray
  :param object_to_copy | int ObjectToCopy
  :param data_container_to_copy | DataArrayPath DataContainerToCopy
  :param attribute_matrix_to_copy | DataArrayPath AttributeMatrixToCopy
  :param attribute_array_to_copy | DataArrayPath AttributeArrayToCopy
  :param copied_object_name | QString CopiedObjectName
  :param observer | Observer
  """

  filter = simpl.CopyObject()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if object_to_copy is not None:
    filter.ObjectToCopy = object_to_copy
  if data_container_to_copy is not None:
    filter.DataContainerToCopy = data_container_to_copy
  if attribute_matrix_to_copy is not None:
    filter.AttributeMatrixToCopy = attribute_matrix_to_copy
  if attribute_array_to_copy is not None:
    filter.AttributeArrayToCopy = attribute_array_to_copy
  if copied_object_name is not None:
    filter.CopiedObjectName = copied_object_name
  filter.execute()
  return filter.ErrorCode

def create_geometry(data_container_array, geometry_type = None, data_container_name = None, shared_vertex_list_array_path0 = None, shared_vertex_list_array_path1 = None, shared_vertex_list_array_path2 = None, shared_vertex_list_array_path3 = None, shared_vertex_list_array_path4 = None, shared_edge_list_array_path = None, shared_tri_list_array_path = None, shared_quad_list_array_path = None, shared_tet_list_array_path = None, x_bounds_array_path = None, y_bounds_array_path = None, z_bounds_array_path = None, dimensions = None, origin = None, spacing = None, image_cell_attribute_matrix_name = None, rect_grid_cell_attribute_matrix_name = None, vertex_attribute_matrix_name0 = None, vertex_attribute_matrix_name1 = None, vertex_attribute_matrix_name2 = None, vertex_attribute_matrix_name3 = None, vertex_attribute_matrix_name4 = None, edge_attribute_matrix_name = None, face_attribute_matrix_name0 = None, face_attribute_matrix_name1 = None, tet_cell_attribute_matrix_name = None, treat_warnings_as_errors = None, array_handling = None, observer = None) -> int:
  """
  CreateGeometry

  :param data_container_array | DataContainerArray
  :param geometry_type | int GeometryType
  :param data_container_name | DataArrayPath DataContainerName
  :param shared_vertex_list_array_path0 | DataArrayPath SharedVertexListArrayPath0
  :param shared_vertex_list_array_path1 | DataArrayPath SharedVertexListArrayPath1
  :param shared_vertex_list_array_path2 | DataArrayPath SharedVertexListArrayPath2
  :param shared_vertex_list_array_path3 | DataArrayPath SharedVertexListArrayPath3
  :param shared_vertex_list_array_path4 | DataArrayPath SharedVertexListArrayPath4
  :param shared_edge_list_array_path | DataArrayPath SharedEdgeListArrayPath
  :param shared_tri_list_array_path | DataArrayPath SharedTriListArrayPath
  :param shared_quad_list_array_path | DataArrayPath SharedQuadListArrayPath
  :param shared_tet_list_array_path | DataArrayPath SharedTetListArrayPath
  :param x_bounds_array_path | DataArrayPath XBoundsArrayPath
  :param y_bounds_array_path | DataArrayPath YBoundsArrayPath
  :param z_bounds_array_path | DataArrayPath ZBoundsArrayPath
  :param dimensions | IntVec3Type Dimensions
  :param origin | FloatVec3Type Origin
  :param spacing | FloatVec3Type Spacing
  :param image_cell_attribute_matrix_name | QString ImageCellAttributeMatrixName
  :param rect_grid_cell_attribute_matrix_name | QString RectGridCellAttributeMatrixName
  :param vertex_attribute_matrix_name0 | QString VertexAttributeMatrixName0
  :param vertex_attribute_matrix_name1 | QString VertexAttributeMatrixName1
  :param vertex_attribute_matrix_name2 | QString VertexAttributeMatrixName2
  :param vertex_attribute_matrix_name3 | QString VertexAttributeMatrixName3
  :param vertex_attribute_matrix_name4 | QString VertexAttributeMatrixName4
  :param edge_attribute_matrix_name | QString EdgeAttributeMatrixName
  :param face_attribute_matrix_name0 | QString FaceAttributeMatrixName0
  :param face_attribute_matrix_name1 | QString FaceAttributeMatrixName1
  :param tet_cell_attribute_matrix_name | QString TetCellAttributeMatrixName
  :param treat_warnings_as_errors | bool TreatWarningsAsErrors
  :param array_handling | bool ArrayHandling
  :param observer | Observer
  """

  filter = simpl.CreateGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if geometry_type is not None:
    filter.GeometryType = geometry_type
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if shared_vertex_list_array_path0 is not None:
    filter.SharedVertexListArrayPath0 = shared_vertex_list_array_path0
  if shared_vertex_list_array_path1 is not None:
    filter.SharedVertexListArrayPath1 = shared_vertex_list_array_path1
  if shared_vertex_list_array_path2 is not None:
    filter.SharedVertexListArrayPath2 = shared_vertex_list_array_path2
  if shared_vertex_list_array_path3 is not None:
    filter.SharedVertexListArrayPath3 = shared_vertex_list_array_path3
  if shared_vertex_list_array_path4 is not None:
    filter.SharedVertexListArrayPath4 = shared_vertex_list_array_path4
  if shared_edge_list_array_path is not None:
    filter.SharedEdgeListArrayPath = shared_edge_list_array_path
  if shared_tri_list_array_path is not None:
    filter.SharedTriListArrayPath = shared_tri_list_array_path
  if shared_quad_list_array_path is not None:
    filter.SharedQuadListArrayPath = shared_quad_list_array_path
  if shared_tet_list_array_path is not None:
    filter.SharedTetListArrayPath = shared_tet_list_array_path
  if x_bounds_array_path is not None:
    filter.XBoundsArrayPath = x_bounds_array_path
  if y_bounds_array_path is not None:
    filter.YBoundsArrayPath = y_bounds_array_path
  if z_bounds_array_path is not None:
    filter.ZBoundsArrayPath = z_bounds_array_path
  if dimensions is not None:
    filter.Dimensions = dimensions
  if origin is not None:
    filter.Origin = origin
  if spacing is not None:
    filter.Spacing = spacing
  if image_cell_attribute_matrix_name is not None:
    filter.ImageCellAttributeMatrixName = image_cell_attribute_matrix_name
  if rect_grid_cell_attribute_matrix_name is not None:
    filter.RectGridCellAttributeMatrixName = rect_grid_cell_attribute_matrix_name
  if vertex_attribute_matrix_name0 is not None:
    filter.VertexAttributeMatrixName0 = vertex_attribute_matrix_name0
  if vertex_attribute_matrix_name1 is not None:
    filter.VertexAttributeMatrixName1 = vertex_attribute_matrix_name1
  if vertex_attribute_matrix_name2 is not None:
    filter.VertexAttributeMatrixName2 = vertex_attribute_matrix_name2
  if vertex_attribute_matrix_name3 is not None:
    filter.VertexAttributeMatrixName3 = vertex_attribute_matrix_name3
  if vertex_attribute_matrix_name4 is not None:
    filter.VertexAttributeMatrixName4 = vertex_attribute_matrix_name4
  if edge_attribute_matrix_name is not None:
    filter.EdgeAttributeMatrixName = edge_attribute_matrix_name
  if face_attribute_matrix_name0 is not None:
    filter.FaceAttributeMatrixName0 = face_attribute_matrix_name0
  if face_attribute_matrix_name1 is not None:
    filter.FaceAttributeMatrixName1 = face_attribute_matrix_name1
  if tet_cell_attribute_matrix_name is not None:
    filter.TetCellAttributeMatrixName = tet_cell_attribute_matrix_name
  if treat_warnings_as_errors is not None:
    filter.TreatWarningsAsErrors = treat_warnings_as_errors
  if array_handling is not None:
    filter.ArrayHandling = array_handling
  filter.execute()
  return filter.ErrorCode

def crop_vertex_geometry(data_container_array, data_container_name = None, cropped_data_container_name = None, x_min = None, y_min = None, z_min = None, x_max = None, y_max = None, z_max = None, observer = None) -> int:
  """
  CropVertexGeometry

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param cropped_data_container_name | DataArrayPath CroppedDataContainerName
  :param x_min | float XMin
  :param y_min | float YMin
  :param z_min | float ZMin
  :param x_max | float XMax
  :param y_max | float YMax
  :param z_max | float ZMax
  :param observer | Observer
  """

  filter = simpl.CropVertexGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cropped_data_container_name is not None:
    filter.CroppedDataContainerName = cropped_data_container_name
  if x_min is not None:
    filter.XMin = x_min
  if y_min is not None:
    filter.YMin = y_min
  if z_min is not None:
    filter.ZMin = z_min
  if x_max is not None:
    filter.XMax = x_max
  if y_max is not None:
    filter.YMax = y_max
  if z_max is not None:
    filter.ZMax = z_max
  filter.execute()
  return filter.ErrorCode

def data_container_reader(data_container_array, input_file = None, overwrite_existing_data_containers = None, input_file_data_container_array_proxy = None, observer = None) -> int:
  """
  DataContainerReader

  :param data_container_array | DataContainerArray
  :param input_file | QString InputFile
  :param overwrite_existing_data_containers | bool OverwriteExistingDataContainers
  :param input_file_data_container_array_proxy | DataContainerArrayProxy InputFileDataContainerArrayProxy
  :param observer | Observer
  """

  filter = simpl.DataContainerReader()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_file is not None:
    filter.InputFile = input_file
  if overwrite_existing_data_containers is not None:
    filter.OverwriteExistingDataContainers = overwrite_existing_data_containers
  if input_file_data_container_array_proxy is not None:
    filter.InputFileDataContainerArrayProxy = input_file_data_container_array_proxy
  filter.execute()
  return filter.ErrorCode

def data_container_writer(data_container_array, output_file = None, write_xdmf_file = None, write_time_series = None, observer = None) -> int:
  """
  DataContainerWriter

  :param data_container_array | DataContainerArray
  :param output_file | QString OutputFile
  :param write_xdmf_file | bool WriteXdmfFile
  :param write_time_series | bool WriteTimeSeries
  :param observer | Observer
  """

  filter = simpl.DataContainerWriter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if output_file is not None:
    filter.OutputFile = output_file
  if write_xdmf_file is not None:
    filter.WriteXdmfFile = write_xdmf_file
  if write_time_series is not None:
    filter.WriteTimeSeries = write_time_series
  filter.execute()
  return filter.ErrorCode

def execute_process(data_container_array, arguments = None, observer = None) -> int:
  """
  ExecuteProcess

  :param data_container_array | DataContainerArray
  :param arguments | QString Arguments
  :param observer | Observer
  """

  filter = simpl.ExecuteProcess()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if arguments is not None:
    filter.Arguments = arguments
  filter.execute()
  return filter.ErrorCode

def extract_attribute_arrays_from_geometry(data_container_array, data_container_name = None, shared_vertex_list_array_path0 = None, shared_vertex_list_array_path1 = None, shared_vertex_list_array_path2 = None, shared_vertex_list_array_path3 = None, shared_vertex_list_array_path4 = None, shared_edge_list_array_path = None, shared_tri_list_array_path = None, shared_quad_list_array_path = None, shared_tet_list_array_path = None, x_bounds_array_path = None, y_bounds_array_path = None, z_bounds_array_path = None, observer = None) -> int:
  """
  ExtractAttributeArraysFromGeometry

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param shared_vertex_list_array_path0 | DataArrayPath SharedVertexListArrayPath0
  :param shared_vertex_list_array_path1 | DataArrayPath SharedVertexListArrayPath1
  :param shared_vertex_list_array_path2 | DataArrayPath SharedVertexListArrayPath2
  :param shared_vertex_list_array_path3 | DataArrayPath SharedVertexListArrayPath3
  :param shared_vertex_list_array_path4 | DataArrayPath SharedVertexListArrayPath4
  :param shared_edge_list_array_path | DataArrayPath SharedEdgeListArrayPath
  :param shared_tri_list_array_path | DataArrayPath SharedTriListArrayPath
  :param shared_quad_list_array_path | DataArrayPath SharedQuadListArrayPath
  :param shared_tet_list_array_path | DataArrayPath SharedTetListArrayPath
  :param x_bounds_array_path | DataArrayPath XBoundsArrayPath
  :param y_bounds_array_path | DataArrayPath YBoundsArrayPath
  :param z_bounds_array_path | DataArrayPath ZBoundsArrayPath
  :param observer | Observer
  """

  filter = simpl.ExtractAttributeArraysFromGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if shared_vertex_list_array_path0 is not None:
    filter.SharedVertexListArrayPath0 = shared_vertex_list_array_path0
  if shared_vertex_list_array_path1 is not None:
    filter.SharedVertexListArrayPath1 = shared_vertex_list_array_path1
  if shared_vertex_list_array_path2 is not None:
    filter.SharedVertexListArrayPath2 = shared_vertex_list_array_path2
  if shared_vertex_list_array_path3 is not None:
    filter.SharedVertexListArrayPath3 = shared_vertex_list_array_path3
  if shared_vertex_list_array_path4 is not None:
    filter.SharedVertexListArrayPath4 = shared_vertex_list_array_path4
  if shared_edge_list_array_path is not None:
    filter.SharedEdgeListArrayPath = shared_edge_list_array_path
  if shared_tri_list_array_path is not None:
    filter.SharedTriListArrayPath = shared_tri_list_array_path
  if shared_quad_list_array_path is not None:
    filter.SharedQuadListArrayPath = shared_quad_list_array_path
  if shared_tet_list_array_path is not None:
    filter.SharedTetListArrayPath = shared_tet_list_array_path
  if x_bounds_array_path is not None:
    filter.XBoundsArrayPath = x_bounds_array_path
  if y_bounds_array_path is not None:
    filter.YBoundsArrayPath = y_bounds_array_path
  if z_bounds_array_path is not None:
    filter.ZBoundsArrayPath = z_bounds_array_path
  filter.execute()
  return filter.ErrorCode

def extract_component_as_array(data_container_array, selected_array_path = None, comp_number = None, new_array_array_name = None, observer = None) -> int:
  """
  ExtractComponentAsArray

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param comp_number | int CompNumber
  :param new_array_array_name | QString NewArrayArrayName
  :param observer | Observer
  """

  filter = simpl.ExtractComponentAsArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if comp_number is not None:
    filter.CompNumber = comp_number
  if new_array_array_name is not None:
    filter.NewArrayArrayName = new_array_array_name
  filter.execute()
  return filter.ErrorCode

def feature_count_decision(data_container_array, feature_ids_array_path = None, max_grains = None, observer = None) -> int:
  """
  FeatureCountDecision

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param max_grains | int MaxGrains
  :param observer | Observer
  """

  filter = simpl.FeatureCountDecision()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if max_grains is not None:
    filter.MaxGrains = max_grains
  filter.execute()
  return filter.ErrorCode

def feature_data_csv_writer(data_container_array, cell_feature_attribute_matrix_path = None, feature_data_file = None, write_neighbor_list_data = None, delimiter_choice = None, write_num_features_line = None, observer = None) -> int:
  """
  FeatureDataCSVWriter

  :param data_container_array | DataContainerArray
  :param cell_feature_attribute_matrix_path | DataArrayPath CellFeatureAttributeMatrixPath
  :param feature_data_file | QString FeatureDataFile
  :param write_neighbor_list_data | bool WriteNeighborListData
  :param delimiter_choice | int DelimiterChoice
  :param write_num_features_line | bool WriteNumFeaturesLine
  :param observer | Observer
  """

  filter = simpl.FeatureDataCSVWriter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_feature_attribute_matrix_path is not None:
    filter.CellFeatureAttributeMatrixPath = cell_feature_attribute_matrix_path
  if feature_data_file is not None:
    filter.FeatureDataFile = feature_data_file
  if write_neighbor_list_data is not None:
    filter.WriteNeighborListData = write_neighbor_list_data
  if delimiter_choice is not None:
    filter.DelimiterChoice = delimiter_choice
  if write_num_features_line is not None:
    filter.WriteNumFeaturesLine = write_num_features_line
  filter.execute()
  return filter.ErrorCode

def find_derivatives(data_container_array, selected_array_path = None, derivatives_array_path = None, observer = None) -> int:
  """
  FindDerivatives

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param derivatives_array_path | DataArrayPath DerivativesArrayPath
  :param observer | Observer
  """

  filter = simpl.FindDerivatives()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if derivatives_array_path is not None:
    filter.DerivativesArrayPath = derivatives_array_path
  filter.execute()
  return filter.ErrorCode

def generate_color_table(data_container_array, selected_preset_name = None, selected_preset_control_points = None, selected_data_array_path = None, rgb_array_name = None, observer = None) -> int:
  """
  GenerateColorTable

  :param data_container_array | DataContainerArray
  :param selected_preset_name | QString SelectedPresetName
  :param selected_preset_control_points | QJsonArray SelectedPresetControlPoints
  :param selected_data_array_path | DataArrayPath SelectedDataArrayPath
  :param rgb_array_name | QString RgbArrayName
  :param observer | Observer
  """

  filter = simpl.GenerateColorTable()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_preset_name is not None:
    filter.SelectedPresetName = selected_preset_name
  if selected_preset_control_points is not None:
    filter.SelectedPresetControlPoints = selected_preset_control_points
  if selected_data_array_path is not None:
    filter.SelectedDataArrayPath = selected_data_array_path
  if rgb_array_name is not None:
    filter.RgbArrayName = rgb_array_name
  filter.execute()
  return filter.ErrorCode

def generate_tilt_series(data_container_array, rotation_axis = None, rotation_limits = None, spacing = None, input_data_array_path = None, output_prefix = None, observer = None) -> int:
  """
  GenerateTiltSeries

  :param data_container_array | DataContainerArray
  :param rotation_axis | int RotationAxis
  :param rotation_limits | float RotationLimits
  :param spacing | float Spacing
  :param input_data_array_path | DataArrayPath InputDataArrayPath
  :param output_prefix | QString OutputPrefix
  :param observer | Observer
  """

  filter = simpl.GenerateTiltSeries()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if rotation_axis is not None:
    filter.RotationAxis = rotation_axis
  if rotation_limits is not None:
    filter.RotationLimits = rotation_limits
  if spacing is not None:
    filter.Spacing = spacing
  if input_data_array_path is not None:
    filter.InputDataArrayPath = input_data_array_path
  if output_prefix is not None:
    filter.OutputPrefix = output_prefix
  filter.execute()
  return filter.ErrorCode

def import_asci_data_array(data_container_array, created_attribute_array_path = None, scalar_type = None, number_of_components = None, skip_header_lines = None, input_file = None, delimiter = None, observer = None) -> int:
  """
  ImportAsciDataArray

  :param data_container_array | DataContainerArray
  :param created_attribute_array_path | DataArrayPath CreatedAttributeArrayPath
  :param scalar_type | SIMPL::NumericTypes::Type ScalarType
  :param number_of_components | int NumberOfComponents
  :param skip_header_lines | int SkipHeaderLines
  :param input_file | QString InputFile
  :param delimiter | int Delimiter
  :param observer | Observer
  """

  filter = simpl.ImportAsciDataArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if created_attribute_array_path is not None:
    filter.CreatedAttributeArrayPath = created_attribute_array_path
  if scalar_type is not None:
    filter.ScalarType = scalar_type
  if number_of_components is not None:
    filter.NumberOfComponents = number_of_components
  if skip_header_lines is not None:
    filter.SkipHeaderLines = skip_header_lines
  if input_file is not None:
    filter.InputFile = input_file
  if delimiter is not None:
    filter.Delimiter = delimiter
  filter.execute()
  return filter.ErrorCode

def import_hdf5_dataset(data_container_array, hdf5_file_path = None, dataset_import_info_list = None, selected_attribute_matrix = None, observer = None) -> int:
  """
  ImportHDF5Dataset

  :param data_container_array | DataContainerArray
  :param hdf5_file_path | QString HDF5FilePath
  :param dataset_import_info_list | QList<ImportHDF5Dataset::DatasetImportInfo> DatasetImportInfoList
  :param selected_attribute_matrix | DataArrayPath SelectedAttributeMatrix
  :param observer | Observer
  """

  filter = simpl.ImportHDF5Dataset()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if hdf5_file_path is not None:
    filter.HDF5FilePath = hdf5_file_path
  if dataset_import_info_list is not None:
    filter.DatasetImportInfoList = dataset_import_info_list
  if selected_attribute_matrix is not None:
    filter.SelectedAttributeMatrix = selected_attribute_matrix
  filter.execute()
  return filter.ErrorCode

def initialize_data(data_container_array, cell_attribute_matrix_paths = None, x_min = None, y_min = None, z_min = None, x_max = None, y_max = None, z_max = None, init_type = None, random = None, init_value = None, init_range = None, observer = None) -> int:
  """
  InitializeData

  :param data_container_array | DataContainerArray
  :param cell_attribute_matrix_paths | std::vector<DataArrayPath> CellAttributeMatrixPaths
  :param x_min | int XMin
  :param y_min | int YMin
  :param z_min | int ZMin
  :param x_max | int XMax
  :param y_max | int YMax
  :param z_max | int ZMax
  :param init_type | int InitType
  :param random | bool Random
  :param init_value | double InitValue
  :param init_range | FPRangePair InitRange
  :param observer | Observer
  """

  filter = simpl.InitializeData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_attribute_matrix_paths is not None:
    filter.CellAttributeMatrixPaths = cell_attribute_matrix_paths
  if x_min is not None:
    filter.XMin = x_min
  if y_min is not None:
    filter.YMin = y_min
  if z_min is not None:
    filter.ZMin = z_min
  if x_max is not None:
    filter.XMax = x_max
  if y_max is not None:
    filter.YMax = y_max
  if z_max is not None:
    filter.ZMax = z_max
  if init_type is not None:
    filter.InitType = init_type
  if random is not None:
    filter.Random = random
  if init_value is not None:
    filter.InitValue = init_value
  if init_range is not None:
    filter.InitRange = init_range
  filter.execute()
  return filter.ErrorCode

def link_feature_map_to_element_array(data_container_array, cell_feature_attribute_matrix_name = None, selected_cell_array_path = None, active_array_name = None, observer = None) -> int:
  """
  LinkFeatureMapToElementArray

  :param data_container_array | DataContainerArray
  :param cell_feature_attribute_matrix_name | QString CellFeatureAttributeMatrixName
  :param selected_cell_array_path | DataArrayPath SelectedCellArrayPath
  :param active_array_name | QString ActiveArrayName
  :param observer | Observer
  """

  filter = simpl.LinkFeatureMapToElementArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_feature_attribute_matrix_name is not None:
    filter.CellFeatureAttributeMatrixName = cell_feature_attribute_matrix_name
  if selected_cell_array_path is not None:
    filter.SelectedCellArrayPath = selected_cell_array_path
  if active_array_name is not None:
    filter.ActiveArrayName = active_array_name
  filter.execute()
  return filter.ErrorCode

def mask_count_decision(data_container_array, mask_array_path = None, number_of_trues = None, observer = None) -> int:
  """
  MaskCountDecision

  :param data_container_array | DataContainerArray
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param number_of_trues | int NumberOfTrues
  :param observer | Observer
  """

  filter = simpl.MaskCountDecision()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if number_of_trues is not None:
    filter.NumberOfTrues = number_of_trues
  filter.execute()
  return filter.ErrorCode

def move_data(data_container_array, what_to_move = None, data_container_destination = None, attribute_matrix_source = None, attribute_matrix_destination = None, data_array_source = None, observer = None) -> int:
  """
  MoveData

  :param data_container_array | DataContainerArray
  :param what_to_move | int WhatToMove
  :param data_container_destination | DataArrayPath DataContainerDestination
  :param attribute_matrix_source | DataArrayPath AttributeMatrixSource
  :param attribute_matrix_destination | DataArrayPath AttributeMatrixDestination
  :param data_array_source | DataArrayPath DataArraySource
  :param observer | Observer
  """

  filter = simpl.MoveData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if what_to_move is not None:
    filter.WhatToMove = what_to_move
  if data_container_destination is not None:
    filter.DataContainerDestination = data_container_destination
  if attribute_matrix_source is not None:
    filter.AttributeMatrixSource = attribute_matrix_source
  if attribute_matrix_destination is not None:
    filter.AttributeMatrixDestination = attribute_matrix_destination
  if data_array_source is not None:
    filter.DataArraySource = data_array_source
  filter.execute()
  return filter.ErrorCode

def move_multi_data(data_container_array, what_to_move = None, data_container_destination = None, attribute_matrix_sources = None, attribute_matrix_destination = None, data_array_sources = None, observer = None) -> int:
  """
  MoveMultiData

  :param data_container_array | DataContainerArray
  :param what_to_move | int WhatToMove
  :param data_container_destination | DataArrayPath DataContainerDestination
  :param attribute_matrix_sources | std::vector<DataArrayPath> AttributeMatrixSources
  :param attribute_matrix_destination | DataArrayPath AttributeMatrixDestination
  :param data_array_sources | std::vector<DataArrayPath> DataArraySources
  :param observer | Observer
  """

  filter = simpl.MoveMultiData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if what_to_move is not None:
    filter.WhatToMove = what_to_move
  if data_container_destination is not None:
    filter.DataContainerDestination = data_container_destination
  if attribute_matrix_sources is not None:
    filter.AttributeMatrixSources = attribute_matrix_sources
  if attribute_matrix_destination is not None:
    filter.AttributeMatrixDestination = attribute_matrix_destination
  if data_array_sources is not None:
    filter.DataArraySources = data_array_sources
  filter.execute()
  return filter.ErrorCode

def multi_threshold_objects(data_container_array, destination_array_name = None, selected_thresholds = None, observer = None) -> int:
  """
  MultiThresholdObjects

  :param data_container_array | DataContainerArray
  :param destination_array_name | QString DestinationArrayName
  :param selected_thresholds | ComparisonInputs SelectedThresholds
  :param observer | Observer
  """

  filter = simpl.MultiThresholdObjects()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if destination_array_name is not None:
    filter.DestinationArrayName = destination_array_name
  if selected_thresholds is not None:
    filter.SelectedThresholds = selected_thresholds
  filter.execute()
  return filter.ErrorCode

def multi_threshold_objects2(data_container_array, destination_array_name = None, selected_thresholds = None, observer = None) -> int:
  """
  MultiThresholdObjects2

  :param data_container_array | DataContainerArray
  :param destination_array_name | QString DestinationArrayName
  :param selected_thresholds | ComparisonInputsAdvanced SelectedThresholds
  :param observer | Observer
  """

  filter = simpl.MultiThresholdObjects2()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if destination_array_name is not None:
    filter.DestinationArrayName = destination_array_name
  if selected_thresholds is not None:
    filter.SelectedThresholds = selected_thresholds
  filter.execute()
  return filter.ErrorCode

def pipeline_annotation(data_container_array, summary = None, observer = None) -> int:
  """
  PipelineAnnotation

  :param data_container_array | DataContainerArray
  :param summary | QString Summary
  :param observer | Observer
  """

  filter = simpl.PipelineAnnotation()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if summary is not None:
    filter.Summary = summary
  filter.execute()
  return filter.ErrorCode

def post_slack_message(data_container_array, slack_user = None, slack_url = None, slack_message = None, warnings_as_error = None, observer = None) -> int:
  """
  PostSlackMessage

  :param data_container_array | DataContainerArray
  :param slack_user | QString SlackUser
  :param slack_url | QString SlackUrl
  :param slack_message | QString SlackMessage
  :param warnings_as_error | bool WarningsAsError
  :param observer | Observer
  """

  filter = simpl.PostSlackMessage()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if slack_user is not None:
    filter.SlackUser = slack_user
  if slack_url is not None:
    filter.SlackUrl = slack_url
  if slack_message is not None:
    filter.SlackMessage = slack_message
  if warnings_as_error is not None:
    filter.WarningsAsError = warnings_as_error
  filter.execute()
  return filter.ErrorCode

def raw_binary_reader(data_container_array, created_attribute_array_path = None, scalar_type = None, endian = None, number_of_components = None, skip_header_bytes = None, input_file = None, observer = None) -> int:
  """
  RawBinaryReader

  :param data_container_array | DataContainerArray
  :param created_attribute_array_path | DataArrayPath CreatedAttributeArrayPath
  :param scalar_type | SIMPL::NumericTypes::Type ScalarType
  :param endian | int32_t Endian
  :param number_of_components | int32_t NumberOfComponents
  :param skip_header_bytes | uint64_t SkipHeaderBytes
  :param input_file | QString InputFile
  :param observer | Observer
  """

  filter = simpl.RawBinaryReader()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if created_attribute_array_path is not None:
    filter.CreatedAttributeArrayPath = created_attribute_array_path
  if scalar_type is not None:
    filter.ScalarType = scalar_type
  if endian is not None:
    filter.Endian = endian
  if number_of_components is not None:
    filter.NumberOfComponents = number_of_components
  if skip_header_bytes is not None:
    filter.SkipHeaderBytes = skip_header_bytes
  if input_file is not None:
    filter.InputFile = input_file
  filter.execute()
  return filter.ErrorCode

def read_ascii_data(data_container_array, wizard_data = None, observer = None) -> int:
  """
  ReadASCIIData

  :param data_container_array | DataContainerArray
  :param wizard_data | ASCIIWizardData WizardData
  :param observer | Observer
  """

  filter = simpl.ReadASCIIData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if wizard_data is not None:
    filter.WizardData = wizard_data
  filter.execute()
  return filter.ErrorCode

def remove_arrays(data_container_array, data_arrays_to_remove = None, observer = None) -> int:
  """
  RemoveArrays

  :param data_container_array | DataContainerArray
  :param data_arrays_to_remove | DataContainerArrayProxy DataArraysToRemove
  :param observer | Observer
  """

  filter = simpl.RemoveArrays()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_arrays_to_remove is not None:
    filter.DataArraysToRemove = data_arrays_to_remove
  filter.execute()
  return filter.ErrorCode

def remove_component_from_array(data_container_array, selected_array_path = None, comp_number = None, save_removed_component = None, new_array_array_name = None, reduced_array_array_name = None, observer = None) -> int:
  """
  RemoveComponentFromArray

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param comp_number | int CompNumber
  :param save_removed_component | bool SaveRemovedComponent
  :param new_array_array_name | QString NewArrayArrayName
  :param reduced_array_array_name | QString ReducedArrayArrayName
  :param observer | Observer
  """

  filter = simpl.RemoveComponentFromArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if comp_number is not None:
    filter.CompNumber = comp_number
  if save_removed_component is not None:
    filter.SaveRemovedComponent = save_removed_component
  if new_array_array_name is not None:
    filter.NewArrayArrayName = new_array_array_name
  if reduced_array_array_name is not None:
    filter.ReducedArrayArrayName = reduced_array_array_name
  filter.execute()
  return filter.ErrorCode

def rename_attribute_array(data_container_array, selected_array_path = None, new_array_name = None, observer = None) -> int:
  """
  RenameAttributeArray

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param new_array_name | QString NewArrayName
  :param observer | Observer
  """

  filter = simpl.RenameAttributeArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if new_array_name is not None:
    filter.NewArrayName = new_array_name
  filter.execute()
  return filter.ErrorCode

def rename_attribute_matrix(data_container_array, selected_attribute_matrix_path = None, new_attribute_matrix = None, observer = None) -> int:
  """
  RenameAttributeMatrix

  :param data_container_array | DataContainerArray
  :param selected_attribute_matrix_path | DataArrayPath SelectedAttributeMatrixPath
  :param new_attribute_matrix | QString NewAttributeMatrix
  :param observer | Observer
  """

  filter = simpl.RenameAttributeMatrix()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_attribute_matrix_path is not None:
    filter.SelectedAttributeMatrixPath = selected_attribute_matrix_path
  if new_attribute_matrix is not None:
    filter.NewAttributeMatrix = new_attribute_matrix
  filter.execute()
  return filter.ErrorCode

def rename_data_container(data_container_array, selected_data_container_name = None, new_data_container_name = None, observer = None) -> int:
  """
  RenameDataContainer

  :param data_container_array | DataContainerArray
  :param selected_data_container_name | DataArrayPath SelectedDataContainerName
  :param new_data_container_name | DataArrayPath NewDataContainerName
  :param observer | Observer
  """

  filter = simpl.RenameDataContainer()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_container_name is not None:
    filter.SelectedDataContainerName = selected_data_container_name
  if new_data_container_name is not None:
    filter.NewDataContainerName = new_data_container_name
  filter.execute()
  return filter.ErrorCode

def replace_value_in_array(data_container_array, selected_array = None, remove_value = None, replace_value = None, observer = None) -> int:
  """
  ReplaceValueInArray

  :param data_container_array | DataContainerArray
  :param selected_array | DataArrayPath SelectedArray
  :param remove_value | double RemoveValue
  :param replace_value | double ReplaceValue
  :param observer | Observer
  """

  filter = simpl.ReplaceValueInArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array is not None:
    filter.SelectedArray = selected_array
  if remove_value is not None:
    filter.RemoveValue = remove_value
  if replace_value is not None:
    filter.ReplaceValue = replace_value
  filter.execute()
  return filter.ErrorCode

def required_z_thickness(data_container_array, data_container_selection = None, num_z_voxels = None, preflight_check = None, observer = None) -> int:
  """
  RequiredZThickness

  :param data_container_array | DataContainerArray
  :param data_container_selection | DataArrayPath DataContainerSelection
  :param num_z_voxels | int NumZVoxels
  :param preflight_check | bool PreflightCheck
  :param observer | Observer
  """

  filter = simpl.RequiredZThickness()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_selection is not None:
    filter.DataContainerSelection = data_container_selection
  if num_z_voxels is not None:
    filter.NumZVoxels = num_z_voxels
  if preflight_check is not None:
    filter.PreflightCheck = preflight_check
  filter.execute()
  return filter.ErrorCode

def rotate_sample_ref_frame(data_container_array, cell_attribute_matrix_path = None, rotation_axis = None, rotation_angle = None, slice_by_slice = None, rotation_table = None, rotation_representation_choice = None, observer = None) -> int:
  """
  RotateSampleRefFrame

  :param data_container_array | DataContainerArray
  :param cell_attribute_matrix_path | DataArrayPath CellAttributeMatrixPath
  :param rotation_axis | FloatVec3Type RotationAxis
  :param rotation_angle | float RotationAngle
  :param slice_by_slice | bool SliceBySlice
  :param rotation_table | DynamicTableData RotationTable
  :param rotation_representation_choice | int RotationRepresentationChoice
  :param observer | Observer
  """

  filter = simpl.RotateSampleRefFrame()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_attribute_matrix_path is not None:
    filter.CellAttributeMatrixPath = cell_attribute_matrix_path
  if rotation_axis is not None:
    filter.RotationAxis = rotation_axis
  if rotation_angle is not None:
    filter.RotationAngle = rotation_angle
  if slice_by_slice is not None:
    filter.SliceBySlice = slice_by_slice
  if rotation_table is not None:
    filter.RotationTable = rotation_table
  if rotation_representation_choice is not None:
    filter.RotationRepresentationChoice = rotation_representation_choice
  filter.execute()
  return filter.ErrorCode

def scale_volume(data_container_array, data_container_name = None, surface_data_container_name = None, apply_to_voxel_volume = None, apply_to_surface_mesh = None, scale_factor = None, observer = None) -> int:
  """
  ScaleVolume

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param surface_data_container_name | DataArrayPath SurfaceDataContainerName
  :param apply_to_voxel_volume | bool ApplyToVoxelVolume
  :param apply_to_surface_mesh | bool ApplyToSurfaceMesh
  :param scale_factor | FloatVec3Type ScaleFactor
  :param observer | Observer
  """

  filter = simpl.ScaleVolume()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if surface_data_container_name is not None:
    filter.SurfaceDataContainerName = surface_data_container_name
  if apply_to_voxel_volume is not None:
    filter.ApplyToVoxelVolume = apply_to_voxel_volume
  if apply_to_surface_mesh is not None:
    filter.ApplyToSurfaceMesh = apply_to_surface_mesh
  if scale_factor is not None:
    filter.ScaleFactor = scale_factor
  filter.execute()
  return filter.ErrorCode

def set_origin_resolution_image_geom(data_container_array, data_container_name = None, change_origin = None, origin = None, change_resolution = None, spacing = None, observer = None) -> int:
  """
  SetOriginResolutionImageGeom

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param change_origin | bool ChangeOrigin
  :param origin | FloatVec3Type Origin
  :param change_resolution | bool ChangeResolution
  :param spacing | FloatVec3Type Spacing
  :param observer | Observer
  """

  filter = simpl.SetOriginResolutionImageGeom()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if change_origin is not None:
    filter.ChangeOrigin = change_origin
  if origin is not None:
    filter.Origin = origin
  if change_resolution is not None:
    filter.ChangeResolution = change_resolution
  if spacing is not None:
    filter.Spacing = spacing
  filter.execute()
  return filter.ErrorCode

def split_attribute_array(data_container_array, input_array_path = None, split_arrays_suffix = None, observer = None) -> int:
  """
  SplitAttributeArray

  :param data_container_array | DataContainerArray
  :param input_array_path | DataArrayPath InputArrayPath
  :param split_arrays_suffix | QString SplitArraysSuffix
  :param observer | Observer
  """

  filter = simpl.SplitAttributeArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_array_path is not None:
    filter.InputArrayPath = input_array_path
  if split_arrays_suffix is not None:
    filter.SplitArraysSuffix = split_arrays_suffix
  filter.execute()
  return filter.ErrorCode

def write_ascii_data(data_container_array, selected_data_array_paths = None, output_path = None, output_file_path = None, delimiter = None, file_extension = None, max_val_per_line = None, output_style = None, observer = None) -> int:
  """
  WriteASCIIData

  :param data_container_array | DataContainerArray
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param output_path | QString OutputPath
  :param output_file_path | QString OutputFilePath
  :param delimiter | int Delimiter
  :param file_extension | QString FileExtension
  :param max_val_per_line | int MaxValPerLine
  :param output_style | int OutputStyle
  :param observer | Observer
  """

  filter = simpl.WriteASCIIData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  if output_path is not None:
    filter.OutputPath = output_path
  if output_file_path is not None:
    filter.OutputFilePath = output_file_path
  if delimiter is not None:
    filter.Delimiter = delimiter
  if file_extension is not None:
    filter.FileExtension = file_extension
  if max_val_per_line is not None:
    filter.MaxValPerLine = max_val_per_line
  if output_style is not None:
    filter.OutputStyle = output_style
  filter.execute()
  return filter.ErrorCode

def write_triangle_geometry(data_container_array, data_container_selection = None, output_nodes_file = None, output_triangles_file = None, observer = None) -> int:
  """
  WriteTriangleGeometry

  :param data_container_array | DataContainerArray
  :param data_container_selection | DataArrayPath DataContainerSelection
  :param output_nodes_file | QString OutputNodesFile
  :param output_triangles_file | QString OutputTrianglesFile
  :param observer | Observer
  """

  filter = simpl.WriteTriangleGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_selection is not None:
    filter.DataContainerSelection = data_container_selection
  if output_nodes_file is not None:
    filter.OutputNodesFile = output_nodes_file
  if output_triangles_file is not None:
    filter.OutputTrianglesFile = output_triangles_file
  filter.execute()
  return filter.ErrorCode

def breakpoint(data_container_array, observer = None) -> int:
  """
  Breakpoint

  :param data_container_array | DataContainerArray
  :param observer | Observer
  """

  filter = simpl.Breakpoint()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  filter.execute()
  return filter.ErrorCode

def empty_filter(data_container_array, observer = None) -> int:
  """
  EmptyFilter

  :param data_container_array | DataContainerArray
  :param observer | Observer
  """

  filter = simpl.EmptyFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  filter.execute()
  return filter.ErrorCode

def mass_create_data(data_container_array, data_container_name = None, observer = None) -> int:
  """
  MassCreateData

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param observer | Observer
  """

  filter = simpl.MassCreateData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  filter.execute()
  return filter.ErrorCode

